/*
 *  dialog.h -- common declarations for all dialog modules
 *
 *  Savio Lam (lam836@cs.cuhk.hk)
 *  Alessandro Rubini (rubini@ipvvis.unipv.it)
 *     a few others contributed
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h> /* at least "FILE" must be defined */

#define DIALOG_VERSION "0.69" /* RELEASE */

/* exit value for errors */
#define DIALOG_ERROR 128


/*
 * The following structure should embed any information relevant to
 * different dialog flavours. It can be enhanced without compatibility
 * problems, as long as the 0-valued (or NULL) fields get the default
 * treatment and applications are careful to have it declared global
 * or use calloc() to obtain it.
 * clientdata is used by each flavour in a different way -- see the docs.
 *
 * Note that win is a "WINDOW *". However, it is only used internally, and
 * its definition depends on <curses.h> or <ncurses.h>. It's better not to
 * force it, and leave the application free not to include any of them.
 */

typedef struct Dialog_desc {
  void   *win;
  short    type;                 /* type of dialog */
  short    c_wid, c_hei;         /* cfg: positive, 0, negative */
  short    wid,hei;              /* actual data: positive */
  char   *backtitle;
  char   *title;
  char   *prompt;
  short    hlinenum;             /* number of horiz lines */
  short    hline[4];             /* where are they: pos or neg */
  short    buttnum;
  short    selected;             /* which button is preselected */
  struct Dialog_butt *buttons;   /* array */
  void   *clientdata;
  short    xtra;                 /* extra argument (list height or such) */
  } Dialog_desc;


/*
 * A structure to embed button status and trim down button-related code
 */

typedef struct Dialog_butt {
  int c_x, c_y;       /* positive, 0, negative */
  int x, y;           /* the actual ones */
  char *text;
  char *hotkey;       /* points within text */
  short status; /* only one in them must be selected. The first by default */
  short moused; /* a flag, to avoid creating multiple mouse regions on it */
  } Dialog_butt;

enum dialog_button_status {
    DIALOG_B_DISABLED=0,
    DIALOG_B_CHOSEN,
    DIALOG_B_ENABLED
};

/*
 * The internal data structure is a double-linked list.
 */

typedef struct Dialog_list {
  short selected, index; /* index is 0-based */
  char *tag;
  char *text;
  char *hotkey;
  struct Dialog_list *next;
  struct Dialog_list *prev;
  } Dialog_list; 


/*
 * These functions provide conversion to the list used internally.
 */

Dialog_list *dialog_a2l(int argc, char **argv, int step);
Dialog_list *dialog_f2l(FILE *f, int step);


/*
 * Utility functions
 */
void     dialog_init (void);
void     dialog_end (void);
void     dialog_clear (void);
int      dialog_defaultbuttons (struct Dialog_desc *desc, char *opts);


/*
 * These are the engines for the different flavours of dialog
 */

int     dialog_run (int flavour, Dialog_desc *desc);

int     dialog_simple           (struct Dialog_desc *desc);
#define dialog_yesno(desc)     dialog_simple(desc)
#define dialog_msgbox(desc)    dialog_simple(desc)
#define dialog_infobox(desc)   dialog_simple(desc)
int     dialog_list             (struct Dialog_desc *desc);
#define dialog_checklist       dialog_list(desc)
#define dialog_radiolist(desc) dialog_list(desc);
#define dialog_menu(desc)      dialog_list(desc);
int     dialog_inputbox         (struct Dialog_desc *desc);
int     dialog_textbox          (struct Dialog_desc *desc);
int     dialog_gauge            (struct Dialog_desc *desc);

extern char dialog_input_result[];

/*
 * The gauge *can* be built by pieces. If desc->win is NULL,
 * then dialog_gauge_update() will initialize the dialog as well.
 * dialog_gauge_init() doesn't exist because
 * it's sufficient to call dialog_init()
 */

int dialog_gauge_update (Dialog_desc *desc, char *prompt, int percent);
int dialog_gauge_end (Dialog_desc *desc);

/*
 * This values select between different flavours
 */
enum dialog_flavour {
  DIALOG_YESNO = 'y',
  DIALOG_MSG   = 'k',
  DIALOG_INFO  = 'i',
  DIALOG_RADIO = 'r',
  DIALOG_CHECK = 'c',
  DIALOG_MENU  = 'm',
  DIALOG_TEXT  = 't',
  DIALOG_INPUT = 'q',
  DIALOG_GAUGE = 'g'
};



/**** Local Variables:  ****/
/**** tab-width:8       ****/
/**** c-indent-level:4  ****/
/**** c-label-offset:-2 ****/
/**** End:              ****/





